#include <SoftwareSerial.h>
#include <Adafruit_Fingerprint.h>
#include <fpm.h>
#include <SD.h>
#include <SPI.h>
/* Send a fingerprint image to a PC.
 *
 * This example should be executed alongside the Python script in the extras folder,
 * which will run on the PC to receive and assemble the fingerprint image.
 */

/*  pin #2 is Arduino RX <==> Sensor TX
 *  pin #3 is Arduino TX <==> Sensor RX
 */
#define NOTE_D7  2349
#define NOTE_A4  440


#define IMG_WIDTH 256
#define IMG_HEIGHT 286
#define IMG_DEPTH 8

SoftwareSerial fserial(2, 3);
FPM finger(&fserial);
FPMSystemParams params;

/* for convenience */
#define turn_green_led_on() do { DDRD |= (1 << PD5); PORTD |= (1 << PD5);} while(0)
#define turn_green_led_off() do { DDRD &= ~(1 << PD5); PORTD &= ~(1 << PD5);} while(0)
#define PRINTF_BUF_SZ   60

char printfBuf[PRINTF_BUF_SZ];

struct __attribute__((packed)) BMPHeader {
  char signature[2];
  uint32_t fileSize;
  uint32_t reserved1;
  uint32_t dataOffset;
  uint32_t infoHeaderSize;
  int32_t width;
  int32_t height;
  uint16_t planes;
  uint16_t bitCount;
  uint32_t compression;
  uint32_t imageSize;
  int32_t xPixelsPerMeter;
  int32_t yPixelsPerMeter;
  uint32_t colorsUsed;
  uint32_t importantColors;

  static BMPHeader create(int width, int height, int depth, bool includePalette) {
    BMPHeader header;
    memset(&header, 0, sizeof(header));

    // Define header value
    strcpy(header.signature, "BM");

    uint32_t byteWidth = ((depth * width + 31) / 32) * 4;
    uint32_t numColors = 2 << depth;
    uint32_t bmpPaletteSize = (includePalette) ? numColors * sizeof(unsigned char) * 2 : 0;
    header.fileSize = sizeof(BMPHeader) + bmpPaletteSize + byteWidth * height;
    header.dataOffset = sizeof(BMPHeader) + bmpPaletteSize;
    header.infoHeaderSize = 40;
    header.width = width;
    header.height = -height; // Invert height for bottom-to-top storage
    header.planes = 1;
    header.bitCount = depth;
    header.compression = 0; // BI_RGB
    header.imageSize = byteWidth * height;
    header.xPixelsPerMeter = 2835;
    header.yPixelsPerMeter = 2835;

    return header;
  }
};

int16_t id;

void setup()
{
    DDRB &= (1 << PB2);
    PORTB |= (1 << PB2);
    Serial.begin(9600);
    fserial.begin(9600);
    SPI.beginTransaction(SPISettings(8000000, MSBFIRST, SPI_MODE0));
    
    //Serial.println("IMAGE-TO-PC example");

    if (finger.begin()) {
        finger.readParams(&params);
        getFreeId(&id);
        // Serial.println("Found fingerprint sensor!");
        // Serial.print("Capacity: "); Serial.println(params.capacity);
        // Serial.print("Packet length: "); Serial.println(FPM::packetLengths[static_cast<uint8_t>(params.packetLen)]);
    } 
    else {
        Serial.println("nf");
        while (1) yield();
    }  
}

char button_state = 1;
uint32_t button_time = millis();
char not_pressed;

void loop()
{
    uint32_t current_time = millis();
    if(id > 100) {
      finger.emptyDatabase();
    }
    not_pressed = PINB & (1 << PB2);
        if (not_pressed != button_state && button_time - millis() > 250)
        {
            button_state = not_pressed;
            button_time = millis();
            if(!not_pressed) {
              while(checkMatch() == -1) ;
              imageToPc();
            }
            else {
              while (! getFingerprintEnroll() );
              id++;
            }

        }


    // while (1) yield();
}

uint32_t imageToPc(void)
{
    
    FPMStatus status;
    
    /* Take a snapshot of the finger */
    Serial.println("gib");
    
    do {
        
        status = finger.getImage();
        
        switch (status) 
        {
            case FPMStatus::OK:
                Serial.println("Took img");
                break;
                
            case FPMStatus::NOFINGER:
                Serial.println(".");
                break;
                
            default:
                /* allow retries even when an error happens */
                snprintf(printfBuf, PRINTF_BUF_SZ, "err 0x%X", static_cast<uint16_t>(status));
                Serial.println(printfBuf);
                break;
        }
        
        yield();
    }
    while (status != FPMStatus::OK);
    
    /* Initiate the image transfer */
    status = finger.downloadImage();
    
    switch (status) 
    {
        case FPMStatus::OK:
            Serial.println("to_sd");
            break;
            
        default:
            snprintf(printfBuf, PRINTF_BUF_SZ, "err 0x%X", static_cast<uint16_t>(status));
            Serial.println(printfBuf);
            return 0;
    }

    /* Send some arbitrary signature to the PC, to indicate the start of the image stream */
    // Serial.write(0xAA);
    
    uint32_t totalRead = 0;
    uint16_t readLen = 130;
    
    /* Now, the sensor will send us the image from its image buffer, one packet at a time.
     * We will stream it directly to Serial. */
    bool readComplete = false;
    if (!SD.begin()) {
      while (1);
    }
    char filename[10];
    snprintf_P(filename, 10, PSTR("fg%hu.bmp"), id);
    if(SD.exists(filename)) {
      SD.remove(filename);
    }
    File img_on_sd = SD.open(filename, O_WRITE | O_CREAT);
    writeBMPHeader(&img_on_sd);
    while (!readComplete) 
    {
        //not_pressed = PINB & (1 << PB2);
        bool ret = finger.readDataPacket(NULL, &img_on_sd, &readLen, &readComplete);
        if (!ret)
        {
            snprintf_P(printfBuf, PRINTF_BUF_SZ, PSTR("readDataPacket(): failed after reading %u bytes"), totalRead);
            Serial.println(printfBuf);
            return 0;
        }
        totalRead += readLen;
        
        yield();
    }
    
    Serial.println();
    Serial.print(totalRead); Serial.println(" bytes tx'd.");
    img_on_sd.close();
    for(int i = 0; i < 3; i++) {
      turn_green_led_on();
      delay(1500);
      turn_green_led_off();
      delay(750);
    }
    return totalRead;
}

void writeBMPHeader(File* destFile) {
    BMPHeader header = BMPHeader::create(IMG_WIDTH, IMG_HEIGHT, IMG_DEPTH, true);
    destFile->write((uint8_t *) &header, sizeof(BMPHeader)/sizeof(uint8_t) );
    for(uint16_t i = 0; i < (1 << IMG_DEPTH); i++) {
      char temp[4];
      temp[0] = temp[1] = temp[2] = temp[3] = i;
      destFile->write(temp, 4);
    }
}

uint8_t getFingerprintEnroll() {
  int p = -1;
  Serial.print("Waiting"); Serial.println(id);
  Adafruit_Fingerprint ada_finger = Adafruit_Fingerprint(&fserial);
  while (p != FINGERPRINT_OK) {
    p = ada_finger.getImage();
    switch (p) {
    case FINGERPRINT_OK:
      Serial.println("took img");
      break;
    case FINGERPRINT_NOFINGER:
      Serial.print(".");
      break;
    case FINGERPRINT_PACKETRECIEVEERR:
      Serial.println("com err");
      break;
    case FINGERPRINT_IMAGEFAIL:
      Serial.println(":(");
      break;
    default:
      Serial.println(":(");
      break;
    }
  }

  // OK success!

  p = ada_finger.image2Tz(1);
  switch (p) {
    case FINGERPRINT_OK:
      Serial.println("OK");
      break;
    case FINGERPRINT_IMAGEMESS:
      Serial.println("messy");
      return p;
    case FINGERPRINT_PACKETRECIEVEERR:
      Serial.println("com_err");
      return p;
    case FINGERPRINT_FEATUREFAIL:
      Serial.println("fail");
      tone(6, NOTE_D7, 1000/4);
      return p;
    case FINGERPRINT_INVALIDIMAGE:
      Serial.println("invalid");
      tone(6, NOTE_D7, 1000/4);
      return p;
    default:
      Serial.println(":(");
      return p;
  }

  delay(2000);
  p = 0;
  while (p != FINGERPRINT_NOFINGER) {
    p = ada_finger.getImage();
  }
  Serial.print("ID "); Serial.println(id);
  p = -1;
  Serial.println("Repeat");
  while (p != FINGERPRINT_OK) {
    p = ada_finger.getImage();
    switch (p) {
    case FINGERPRINT_OK:
      Serial.println("took img");
      break;
    case FINGERPRINT_NOFINGER:
      Serial.print(".");
      break;
    case FINGERPRINT_PACKETRECIEVEERR:
      Serial.println("com err");
      break;
    case FINGERPRINT_IMAGEFAIL:
      Serial.println("fail");
      tone(6, NOTE_D7, 1000/4);
      break;
    default:
      Serial.println(":(");
      break;
    }
  }

  // OK success!

  p = ada_finger.image2Tz(2);
  switch (p) {
    case FINGERPRINT_OK:
      Serial.println("OK");
      break;
    case FINGERPRINT_IMAGEMESS:
      Serial.println("messy");
      return p;
    case FINGERPRINT_PACKETRECIEVEERR:
      Serial.println("com err");
      return p;
    case FINGERPRINT_FEATUREFAIL:
      Serial.println("invalid");
      tone(6, NOTE_D7, 1000/4);
      return p;
    case FINGERPRINT_INVALIDIMAGE:
      Serial.println("invalid");
      tone(6, NOTE_D7, 1000/4);
      return p;
    default:
      Serial.println(":(");
      return p;
  }

  // OK converted!
  Serial.print("busy for");  Serial.println(id);

  p = ada_finger.createModel();
  if (p == FINGERPRINT_OK) {
    Serial.println("Matched");
    tone(6, NOTE_A4, 1000/8);
    delay(300);
    noTone(6);
    tone(6, NOTE_A4, 1000/8);
  } else if (p == FINGERPRINT_PACKETRECIEVEERR) {
    Serial.println("comm err");
    return p;
  } else if (p == FINGERPRINT_ENROLLMISMATCH) {
    Serial.println("nomatch");
    tone(6, NOTE_D7, 1000/4);
    return p;
  } else {
    Serial.println(":(");
    return p;
  }

  Serial.print("ID "); Serial.println(id);
  p = ada_finger.storeModel(id);
  if (p == FINGERPRINT_OK) {
    Serial.println("Stored!");
  } else if (p == FINGERPRINT_PACKETRECIEVEERR) {
    Serial.println("comm err");
    return p;
  } else if (p == FINGERPRINT_BADLOCATION) {
    Serial.println("badloc");
    return p;
  } else if (p == FINGERPRINT_FLASHERR) {
    Serial.println("flasherr");
    return p;
  } else {
    Serial.println(":(");
    return p;
  }

  return true;
}

int checkMatch() {
  Adafruit_Fingerprint ada_finger = Adafruit_Fingerprint(&fserial);
  uint8_t p = ada_finger.getImage();
  if (p != FINGERPRINT_OK)  {
    return -1;
  }
  p = ada_finger.image2Tz();
  if (p != FINGERPRINT_OK)  {
    return -1;
  }
  p = ada_finger.fingerFastSearch();
  if (p != FINGERPRINT_OK)  {
    return -1;
  }
  // found a match!

  Serial.print("Found#"); Serial.print(ada_finger.fingerID);
  Serial.print("confidence "); Serial.println(ada_finger.confidence);
  return ada_finger.fingerID;
}

bool getFreeId(int16_t * fid) 
{
    for (int page = 0; page < (params.capacity / FPM_TEMPLATES_PER_PAGE) + 1; page++) 
    {
        FPMStatus status = finger.getFreeIndex(page, fid);
        
        switch (status) 
        {
            case FPMStatus::OK:
                if (*fid != -1) {
                    Serial.print("Free#");
                    Serial.println(*fid);
                    return true;
                }
                break;
                
            default:
                snprintf(printfBuf, PRINTF_BUF_SZ, "err 0x%X", page, static_cast<uint16_t>(status));
                Serial.println(printfBuf);
                return false;
        }
        
        yield();
    }
    

    return false;
}

void printDirectory(File dir, int numTabs) {
  while (true) {

    File entry =  dir.openNextFile();
    if (! entry) {
      // no more files
      break;
    }
    for (uint8_t i = 0; i < numTabs; i++) {
      Serial.print('\t');
    }
    Serial.print(entry.name());
    if (entry.isDirectory()) {
      Serial.println("/");
      printDirectory(entry, numTabs + 1);
    } else {
      // files have sizes, directories do not
      Serial.print("\t");
      Serial.println(entry.size(), DEC);
    }
    entry.close();
  }
}
